#include <Adafruit_NeoPixel.h>

#define PIN              2 // Pin for Neo Pixels
#define NUMBER_OF_PIXELS 15 // Number of Pixels 
#define COLORS_PER_THEME 3 // Number of Color in each Theme

Adafruit_NeoPixel strip = Adafruit_NeoPixel(NUMBER_OF_PIXELS, PIN, NEO_GRB + NEO_KHZ800);

byte switchPin = 4; //Pin for Color Change/Brightness Control Button
byte val; int t = 0; byte CurrentBrMode = 1; //Just some Vars

byte MaxBright = 100; //Sets brightness on each cold start When first started 
byte lightMode = 1 ; // Sets which mode the earrings should start up on each time 0-9
byte BrightLow = 40; byte BrightMed = 100; byte BrightHigh = 255; // Sets Brightness Level for each setting (1,2,3)


uint8_t themes[][COLORS_PER_THEME][3] = {   // 3 total themes, how many colors in the themes, 3 colors (r,g,b) in a theme
  {{255, 0, 0}, {60, 60, 60}, {0, 255, 0}},     // Christmas (Red White Green)
  {{255, 0, 0}, {100, 100, 100}, {0, 0, 255}},     // 4th of July (Red White Blue)
  {{180, 180, 0}, {50, 255, 50}, {255, 65, 65}},  // Easter (Pastel Colors)
  {{255, 70, 0}, {255, 170, 0}, {255, 15, 0}},    // Fall Colors (Red Orange Yellow)
  {{0, 0, 255}, {70, 70, 255}, {200, 200, 200}},   // Winter Cold Colors (Blue White)
  {{255, 0, 255}, {0, 255, 0}, {90, 90, 255}},  // Some Colors Mixed (Purple Sky Blue Green
  {{190, 0, 255}, {255, 0, 255}, {255, 0, 210}},// Colors of Purple
  {{70, 70, 255}, {40, 40, 255}, {0, 0, 255}},  // Colors of Sky Blue
  {{0, 255, 0}, {45, 255, 45}, {85, 255, 85}}   // Colors of Green
};


void setup()
{
pinMode(switchPin, INPUT_PULLUP);    // Set the switch pin as input
strip.begin(); // Start NeoPixels
strip.show(); // Initialize all pixels to 'off'
strip.setBrightness(MaxBright); // Sets strip brightness to level requested on startup
}

void flashRandom(byte wait, uint8_t howMany, int theme=0) {   
  for(uint16_t i=0; i < howMany; i++) {
    int c = random(COLORS_PER_THEME);
    int r = themes[theme][c][0];
    int g = themes[theme][c][1];
    int b = themes[theme][c][2];
    int pixel_index = random(strip.numPixels());
    
    for(int x=0; x<5; x++) {   
       int dr = r * (x+1); dr /= 5;
       int dg = g * (x+1); dg /= 5;
       int db = b * (x+1); db /= 5;
       strip.setPixelColor(pixel_index, strip.Color(dr, dg, db));
       strip.show(); delay(wait);
    }
    for(int x=5; x>= 0; x--) {
      int dr = r * x; dr /= 5;
      int dg = g * x; dg /= 5;
      int db = b * x; db /= 5;
      strip.setPixelColor(pixel_index, strip.Color(dr, dg, db));
      strip.show(); delay(wait);
    }  
  }
}


void loop ()
{
if (lightMode == 0) {theaterChaseRainbow(100);} // Spinning effect 
if (lightMode >= 1) {flashRandom(5, 1, (lightMode-1)); flashRandom(5, 3, (lightMode-1)); flashRandom(5, 2, (lightMode-1));} // Twinkle modes
ReadButton();
}


void ReadButton() //Read Button to change effects and if a LONG press then only change the brghtness
{
  val = digitalRead(switchPin);      
  if (val== LOW)
    {
      lightMode++;
      if (lightMode == 10){lightMode = 0;}
      delay(1000);
    
      val = digitalRead(switchPin);      
      if (val== LOW)
      {
        lightMode--; if (lightMode < 0) {lightMode = 9;}
        if (CurrentBrMode == 0) {CurrentBrMode = 1;}
            else if (CurrentBrMode == 1) {CurrentBrMode = 2;}
            else if (CurrentBrMode == 2) {CurrentBrMode = 0;}
        if (CurrentBrMode == 0) {MaxBright = BrightLow;}
        if (CurrentBrMode == 1) {MaxBright = BrightMed;}
        if (CurrentBrMode == 2) {MaxBright = BrightHigh;}
        strip.setBrightness(MaxBright);  
        delay(500);
    }
    }



}


//Theatre-style crawling lights with rainbow effect
void theaterChaseRainbow(uint8_t wait) {
  for (int j=0; j < 256; j++) {     // cycle all 256 colors in the wheel
    for (byte q=0; q < 3; q++) {
        for (byte i=0; i < strip.numPixels(); i=i+3) {
          strip.setPixelColor(i+q, Wheel( (i+j) % 255));    //turn every third pixel on
        }
        strip.show();
        if (digitalRead(switchPin) == LOW) {j=255;}
       
        delay(wait);
       
        for (byte i=0; i < strip.numPixels(); i=i+3) {
          strip.setPixelColor(i+q, 0);        //turn every third pixel off
        
      }
    }
  }
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if(WheelPos < 85) {
   return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  } else if(WheelPos < 170) {
    WheelPos -= 85;
   return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  } else {
   WheelPos -= 170;
   return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
  }
}

